@@ -94,6 +94,15 @@ gcs_group_init_history (gcs_group_t*     group,
                         gcs_seqno_t      seqno,
                         const gu_uuid_t* uuid);
 
+#ifdef GCS_CORE_TESTING
+/*!
+ * Free group nodes. Should not be used directly, exposed only for
+ * unit tests.
+ */
+extern void
+group_nodes_free (gcs_group_t* group);
+#endif // GCS_CORE_TESTING
+
 /*!
  * Free group resources
  */
@@ -150,7 +159,7 @@ gcs_group_handle_act_msg (gcs_group_t*          const group,
                           struct gcs_act_rcvd*  const rcvd,
                           bool commonly_supported_version)
 {
-    long const sender_idx = msg->sender_idx;
+    int  const sender_idx = msg->sender_idx;
     bool const local      = (sender_idx == group->my_idx);
     ssize_t ret;
 
@@ -172,6 +181,7 @@ gcs_group_handle_act_msg (gcs_group_t*          const group,
         assert (ret == rcvd->act.buf_len);
 
         rcvd->act.type = frg->act_type;
+        rcvd->sender_idx = sender_idx;
 
         if (gu_likely(GCS_ACT_TORDERED  == rcvd->act.type &&
                       GCS_GROUP_PRIMARY == group->state   &&
